﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using Microsoft.Extensions.Hosting;
using Microsoft.AspNetCore.Mvc;

namespace FastReportCore.Controllers
{
	[Route("api/[controller]")]
	[ApiController]
	public class ValuesController : ControllerBase
	{
		private readonly IHostEnvironment _env;

		public ValuesController(IHostEnvironment env)
		{
			_env = env;
		}

		[HttpPost("Report")]
		[Route("Report")]
		public ModelFastReport Report([FromBody] ModelDemo value)
		{
			try
			{
				DataSet data1 = LoadData1();
				List<Category> data2 = LoadData2();

				FastReport.Web.WebReport wr = new FastReport.Web.WebReport();
				string frx = Path.Combine(_env.ContentRootPath, "reports", value.file);
				wr.Report.Load(frx);
				wr.Report.RegisterData(data1, "NorthWind");
				wr.Report.RegisterData(data2, "Categories BusinessObject");
				foreach (FastReport.Data.DataSourceBase item in wr.Report.Dictionary.DataSources)
				{
					item.Enabled = true;
				}
				string basePath = Request.PathBase.Value;
				return new ModelFastReport() { id = basePath, html = FastReportHtml(basePath, wr.ID, wr.RenderSync().Value) };
			}
			catch (Exception ex)
			{
				return new ModelFastReport() { id = "", html = ex.ToString() };
			}
		}

		private string FastReportHtml(string basePath, string token, string htmlString)
		{
			string oldCssOutline = string.Format(".fr{0}-outline {{", token);
			string newCssOutline = string.Concat(oldCssOutline, "color: black;");
			string oldAjax = "container.outerHTML = xhr.responseText";
			string newAjax = string.Format("{0}.replace(\"{1}\", \"{2}\").replace(/\\/_fr/g, \"{3}/_fr\")", oldAjax, oldCssOutline, newCssOutline, basePath);
			string oldCssProgress = "style=\"min-height:200px\">";
			string newCssProgress = string.Format("><img src=\"{0}/progress.gif\">", basePath);
			string OldCssSpinner = string.Format(".fr{0}-spinner {{", token);
			string NewCssSpinner = string.Concat(OldCssSpinner, "display: none;");
			string oldCssBackgroundColor = "background-color: white;";
			return htmlString.Replace("/_fr", basePath + "/_fr").Replace(oldCssBackgroundColor, "").Replace(oldCssProgress, newCssProgress).Replace(OldCssSpinner, NewCssSpinner).Replace(oldAjax, newAjax);
		}

		private DataSet LoadData1()
		{
			string xmlFile = Path.Combine(_env.ContentRootPath, "reports", "nwind.xml");
			DataSet dataSet = new DataSet();
			dataSet.ReadXml(xmlFile);
			return dataSet;
		}

		private List<Category> LoadData2()
		{
			List<Category> businessObject = new List<Category>();
			Category category = new Category("Beverages", "Soft drinks, coffees, teas, beers");
			category.Products.Add(new Product("Chai", 18m));
			category.Products.Add(new Product("Chang", 19m));
			category.Products.Add(new Product("Ipoh coffee", 46m));
			businessObject.Add(category);
			category = new Category("Confections", "Desserts, candies, and sweet breads");
			category.Products.Add(new Product("Chocolade", 12.75m));
			category.Products.Add(new Product("Scottish Longbreads", 12.5m));
			category.Products.Add(new Product("Tarte au sucre", 49.3m));
			businessObject.Add(category);
			category = new Category("Seafood", "Seaweed and fish");
			category.Products.Add(new Product("Boston Crab Meat", 18.4m));
			category.Products.Add(new Product("Red caviar", 15m));
			businessObject.Add(category);
			return businessObject;
		}

		public class ModelFastReport
		{
			public string id { get; set; }
			public string html { get; set; }
		}

		public class ModelDemo
		{
			public string file { get; set; }
			public string label { get; set; }
		}

		public class Category
		{
			private string FName;

			private string FDescription;

			private List<Product> FProducts;

			public string Name
			{
				get
				{
					return this.FName;
				}
			}

			public string Description
			{
				get
				{
					return this.FDescription;
				}
			}

			public List<Product> Products
			{
				get
				{
					return this.FProducts;
				}
			}

			public Category(string name, string description)
			{
				this.FName = name;
				this.FDescription = description;
				this.FProducts = new List<Product>();
			}
		}

		public class Product
		{
			private string FName;

			private decimal FUnitPrice;

			public string Name
			{
				get
				{
					return this.FName;
				}
			}

			public decimal UnitPrice
			{
				get
				{
					return this.FUnitPrice;
				}
			}

			public Product(string name, decimal unitPrice)
			{
				this.FName = name;
				this.FUnitPrice = unitPrice;
			}
		}
	}
}